from itertools import ifilterfalse
import os
from django.core.exceptions import SuspiciousOperation


def compose_path(*args):
    iter = ifilterfalse(lambda x: x is None, args)       # removes empty pathes
    path = os.path.normpath(os.path.join(*iter))
    if path.startswith(".."):
        raise SuspiciousOperation(".. in %s not allowed" % path)
    return path



from django.conf.urls import url, patterns
from studiogdo.views import SkelView

""" deprecated
urlpatterns = patterns('',
    url(r'^$', view=SkelView.as_view(), name='studiogdo'),
    url(r'^(?P<entry>\w+).gdo$', view=SkelView.as_view(), name='studiogdo'),
    url(r'^lock/(?P<entry>\w+).gdo$', view=SkelView.as_view(), name='lock'),
    url(r'^unlock/(?P<entry>\w+).gdo$', view=SkelView.as_view(), name='unlock'),
)
""" 



from django import template
from django.utils.safestring import mark_safe
from django.template.loader import get_template, get_template_from_string
from base64 import b64encode as encode, b64decode as decode
from django.conf import settings


register = template.Library()

default_facet_mode = getattr(settings, "STUDIOGDO_FACET_MODE", "dom5")


class SkelNode(template.Node):

    def __init__(self, skeleton, path):
        self.skeleton = skeleton
        self.path = path

    def render(self, context):
        skel = self.skeleton.resolve(context)
        path = self.path.resolve(context) if self.path else self.path
        t = get_template(skel)

        # get HTML renderer
        request = context['request']

        # render facet
        if settings.STUDIOGDO_FACET == "django":
            renderer = request.html_renderer
            facet = renderer.render(path, t.render(context))
        elif settings.STUDIOGDO_FACET == "java":
            api = request.studiogdo_api
            unprocessed_skel = t.render(context)
            facet_mode = context.get("studiogdo_facet_mode", default_facet_mode)
            response = api.post_facet(encode(path), unprocessed_skel, facet_mode)
            facet = unicode(response.content, 'utf-8')
            if facet_mode == "trans":
                rederended_skel = get_template_from_string("{%% load i18n %%}\n%s" % facet)
                facet = rederended_skel.render(context)

        # set trace
        studiogdo_debug = context.get('studiogdo_debug', None)
        if studiogdo_debug or (studiogdo_debug is None and getattr(settings, "STUDIOGDO_DEBUG", None)):
            return mark_safe("<!-- start %s (%s)-->\n" % (skel, path) + facet + "<!-- end %s -->\n" % skel)

        return mark_safe(facet)


@register.tag
def skel(parser, token):
    contents = token.split_contents()
    if len(contents) != 3:
        raise template.TemplateSyntaxError('skel needs at 2 arguments')
    else:
        skeleton = parser.compile_filter(contents[1])
        path = parser.compile_filter(contents[2])
        return SkelNode(skeleton, path)


@register.filter
def b64encode(value):
    return encode(value)

@register.filter
def b64decode(value):
    return decode(value)



# -*- coding: utf-8 -*-
from base64 import b64encode

from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils.module_loading import import_by_path
from django.utils.functional import LazyObject


class PipAdmin(object):
    menu = {}
    accueil_tabs = {}

    def __init__(self):
        """
        verify menu dictionnary
        """
        self.complete_menu()
        self.complete_accueil_tabs()

    def complete_menu(self):
        """
        menu example:
        menu = {'group :  {'label': 'Accueil'},
                          {'label': 'CRM', 'menu': [
                              {'label': 'Sociétés', 'url': 'list', 'path': '/Services(crm)/FilteredSociete', },
                              {'label': 'Contacts', 'url': 'list', 'path': '/Session/User(1)/FilteredClient', },
                              {'separator': True},
                              {'label': 'Actions', 'menu': [
                                  {'label': 'Envoi emails', 'url': 'list', 'path': '/', },
                                  {'label': 'Envoi SMS', 'url': 'list', 'path': '/', },
                              ]},
                            ]}}
        """
        for group, menu_group in self.menu.iteritems():
            for item in menu_group:
                flags = {'label or separator': 'label' in item or 'separator' in item}
                if False in flags.values():
                    raise ValidationError('flag %s missing' % str(flags))

                # default url value (if not sub menu)
                if not 'url' in item and not 'menu' in item and 'label' in item:
                    item['url'] = item['label'].lower()

                if 'path' in item:
                    item['path'] = b64encode(item['path'])

                self.complete_menu_recursive(item.get('menu', []))
            else:
                if group != group.lower():
                    menu_group = self.menu.pop(group)
                    self.menu[group.lower()] = menu_group

    def complete_menu_recursive(self, menu):
        for item in menu:
            flags = {'label or separator': 'label' in item or 'separator' in item}
            if False in flags.values():
                raise ValidationError('flag %s missing' % str(flags))

            # default url value (if not sub menu)
            if not 'url' in item and not 'menu' in item and 'label' in item:
                item['url'] = item['label'].lower()

            if 'path' in item:
                item['path'] = b64encode(item['path'])

            self.complete_menu_recursive(item.get('menu', []))

    def complete_accueil_tabs(self):
        for group in self.accueil_tabs.iterkeys():
            if group != group.lower():
                menu_group = self.accueil_tabs.pop(group)
                self.accueil_tabs[group.lower()] = menu_group


class ConfiguredStorage(LazyObject):
    def _setup(self):
        self._wrapped = import_by_path("pip.admin.%s" % settings.PIP_ADMIN)()


admin = ConfiguredStorage()




# -*- coding: utf-8 -*-
import re
from base64 import b64decode, b64encode

from django.contrib.auth.decorators import login_required
from django.core.context_processors import csrf
from django.core.urlresolvers import reverse
from django.http.response import HttpResponseRedirect
from django.shortcuts import redirect, render_to_response
from django.views.generic import TemplateView
from django.utils.decorators import method_decorator
from django.contrib.auth import authenticate, login as auth_login, logout as auth_logout

from studiogdo.pip import pathpatterns


class PIPView(TemplateView):
    """
    This view checks user is connected to be rendered.
    All templates are in pip folder.
    """

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        # check connected
        api = request.studiogdo_api
        response = api.post_facet(None, '<span data-path="/Session/User(1)"></span>', "dom5")
        facet = unicode(response.content, 'utf-8')
        if not facet:
            data = {}
            data["param1"] = request.user.username
            api.apply_command("/", "Connect", data)

        return super(PIPView, self).dispatch(request, *args, **kwargs)

    def get_template_names(self):
        return 'pip/' + self.template_name


class LoginView(TemplateView):
    template_name = 'pip/login.html'

    def post(self, request, *args, **kwargs):
        username = request.POST['username']
        password = request.POST['password']
        user = authenticate(username=username, password=password)
        if user is not None:
            if user.is_active:
                auth_login(request, user)
                api = request.studiogdo_api
                data = {}
                data["param1"] = username
                api.apply_command("/", "Connect", data)
                redirect_to = request.GET.get('next', reverse('accueil'))
                return redirect(redirect_to)
            else:
                msg = "Vous avez été déconnecté..."
        else:
            msg = "Paramètres de connexion invalides!"
        c = {}
        c["error_msg"] = msg
        c.update(csrf(request))
        return render_to_response(LoginView.template_name, c)

    def get(self, request, *args, **kwargs):
        """ Disconnect from server. """
        auth_logout(request)
        api = request.studiogdo_api
        api.disconnect()
        return super(LoginView, self).get(request, *args, **kwargs)


class AccueilView(PIPView):
    template_name = 'accueil.html'


class LockView(PIPView):
    pass


class UnlockView(PIPView):
    pass


class PIPPathView(PIPView):
    """
    Adds skel and context from pathes.py in kwargs.
    """
    default_template_name = None

    def dispatch(self, request, *args, **kwargs):
        self.encoded_path = kwargs["path"]
        self.path = b64decode(self.encoded_path)
        path_dict = pathpatterns[self.path]
        context = path_dict.get("context", {})
        kwargs.update(context)
        self.template_name = path_dict.get("template_name", self.default_template_name)
        self.path_group = path_dict.get("path_group", {})
        self.skel = path_dict["skel"]
        kwargs["skel"] = self.skel
        return super(PIPPathView, self).dispatch(request, *args, **kwargs)


class PIPListView(PIPPathView):
    """
    Adds container path (data-path) and iterator slot path (slot) in context.
    """
    default_template_name = "studiogdo/pip/list.html"

    def get_context_data(self, **kwargs):
        # add path
        context = super(PIPListView, self).get_context_data(**kwargs)
        data_path, slot = self.path.rsplit("/", 1)
        # context["data_path"] = self.path
        context["data_path"] = data_path
        context["slot"] = slot
        return context


class PIPDetailsView(PIPPathView):
    """
    Adds stencil path (data-path) and return list path (return-path) in context.
    """
    default_template_name = "studiogdo/pip/details.html"

    def _get_return_path(self):
        slot_path = re.search("(?P<path>.*)\(\d+\)$", self.path)
        if slot_path:
            return slot_path.group("path")
        return ""

    def get_context_data(self, **kwargs):
        context = super(PIPDetailsView, self).get_context_data(**kwargs)
        context["data_path"] = self.path
        context["return_path"] = self._get_return_path()
        return context

    def post(self, request, *args, **kwargs):
        params = request.POST
        api = request.studiogdo_api
        api.post_empty(self.encoded_path, params)
        save_button = params.get("save", None)
        if save_button == "details":
            return HttpResponseRedirect(reverse("details", args=[self.encoded_path]))
        else:
            return HttpResponseRedirect(reverse("list", args=[b64encode(self._get_return_path())]))



from django.conf import settings


def include_studiogdo_address(request):
    return {
        'studiogdo_host': settings.STUDIOGDO_HOST,
        'studiogdo_servlet': settings.STUDIOGDO_SERVLET,
        'studiogdo_debug': settings.STUDIOGDO_DEBUG or False,
    }



import os
from setuptools import setup, find_packages

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-studiogdo',
    version='0.2.31',
    packages=find_packages(),
    include_package_data=True,
    license='BSD License',  # example license
    description='Django implementation of StudioGdo MVC extension.',
    long_description=README,
    url='http://www.coworks.pro/',
    author='Guillaume Doumenc',
    author_email='gdoumenc@coworks.pro',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'beautifulsoup4>=4.3.2',
        'html5lib>=0.999',
        'ipdb>=0.8',
        'requests>=2.3.0',
    ],
)
